@@ -21,6 +21,8 @@
 
 #include "config.h"
 
+#ifndef TRACKER_DISABLE_DEPRECATED
+
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -56,6 +58,13 @@
  * either by storing their data or by querying it. They are also not
  * limited to their application's data. Other data mined by other
  * applications is also available in some cases.
+ *
+ * Deprecated: 0.10: This whole library has been replaced by libtracker-sparql
+ * in its entirity to provide a cleaner and much more usable API using GLib's
+ * asynchronous and cancellable APIS and a foundation Cursor API shared by both
+ * a direct and d-bus approach. Additionally all escaping and SPARQL building
+ * functionality has moved to this new library and is shared across the Tracker
+ * platform.
  **/
 
 /**
@@ -163,13 +172,13 @@ typedef struct {
 
 struct TrackerResultIterator {
 #ifdef HAVE_DBUS_FD_PASSING
-	char *buffer;
-	int buffer_index;
-	long buffer_size;
+	gchar *buffer;
+	gint buffer_index;
+	gssize buffer_size;
 
 	guint n_columns;
-	int *offsets;
-	char *data;
+	gint *offsets;
+	gchar *data;
 #else  /* HAVE_DBUS_FD_PASSING */
 	GPtrArray *results;
 	gint current_row;
@@ -190,15 +199,10 @@ typedef struct {
 	guint request_id;
 	FastOperationType operation_type;
 
-	GInputStream *input_stream;
-	GOutputStream *output_stream;
 	GCancellable *cancellable;
 
 	DBusPendingCall *dbus_call;
 
-	TrackerResultIterator *result_iterator;
-	gboolean iterator_returned;
-
 	union {
 		TrackerReplyGPtrArray gptrarray_callback;
 		TrackerReplyVoid void_callback;
@@ -342,19 +346,10 @@ fast_async_data_free (gpointer data)
 	FastAsyncData *fad = data;
 
 	if (fad) {
-		if (fad->result_iterator && !fad->iterator_returned) {
-			tracker_result_iterator_free (fad->result_iterator);
-		}
-
 		if (fad->cancellable) {
 			g_object_unref (fad->cancellable);
 		}
 
-		if (fad->dbus_call) {
-			dbus_pending_call_cancel (fad->dbus_call);
-			dbus_pending_call_unref (fad->dbus_call);
-		}
-
 		if (fad->client) {
 			g_object_unref (fad->client);
 		}
@@ -366,10 +361,7 @@ fast_async_data_free (gpointer data)
 static FastAsyncData *
 fast_async_data_new (TrackerClient     *client,
                      FastOperationType  operation_type,
-                     GInputStream      *input_stream,
-                     GOutputStream     *output_stream,
                      GCancellable      *cancellable,
-                     DBusPendingCall   *dbus_call,
                      gpointer           user_data)
 {
 	FastAsyncData *data;
@@ -379,11 +371,7 @@ fast_async_data_new (TrackerClient     *client,
 	data->client = g_object_ref (client);
 	data->request_id = fast_pending_call_new (client, cancellable, data);
 	data->operation_type = operation_type;
-	data->result_iterator = g_slice_new0 (TrackerResultIterator);
-	data->input_stream = input_stream;
-	data->output_stream = output_stream;
 	data->cancellable = cancellable;
-	data->dbus_call = dbus_call;
 	data->user_data = user_data;
 
 	return data;
@@ -650,82 +638,52 @@ iterator_buffer_read_int (TrackerResultIterator *iterator)
 }
 
 static void
-callback_iterator (GObject      *source_object,
-                   GAsyncResult *result,
-                   gpointer      user_data)
+callback_iterator (void     *buffer,
+                   gssize    buffer_size,
+                   GError   *error,
+                   gpointer  user_data)
 {
 	TrackerClientPrivate *private;
-	DBusMessage *reply = NULL;
-	GError *error = NULL;
 	FastAsyncData *fad;
 	TrackerResultIterator *iterator;
-	GInputStream *base_input_stream;
 
-	/* Clean up pending calls */
 	fad = user_data;
 
 	private = TRACKER_CLIENT_GET_PRIVATE (fad->client);
 	g_hash_table_remove (private->fast_pending_calls,
 	                     GUINT_TO_POINTER (fad->request_id));
 
-	/* Reset the iterator internal state */
-	iterator = fad->result_iterator;
-
-	iterator->buffer_size = g_output_stream_splice_finish (fad->output_stream,
-	                                                       result,
-	                                                       &error);
-	iterator->buffer = g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (fad->output_stream));
-	iterator->buffer_index = 0;
-
-	/* Clean up streams */
-	base_input_stream = g_filter_input_stream_get_base_stream (G_FILTER_INPUT_STREAM (fad->input_stream));
-	g_object_unref (fad->input_stream);
-	g_object_unref (fad->output_stream);
-	g_object_unref (base_input_stream);
 
 	/* Check for errors */
 	if (G_LIKELY (!error)) {
-		/* Wait for any current d-bus call to finish */
-		dbus_pending_call_block (fad->dbus_call);
+		iterator = g_slice_new0 (TrackerResultIterator);
 
-		/* Check we didn't get an error */
-		reply = dbus_pending_call_steal_reply (fad->dbus_call);
+		iterator->buffer = buffer;
+		iterator->buffer_size = buffer_size;
+		iterator->buffer_index = 0;
 
-		if (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {
-			DBusError dbus_error;
+		(* fad->iterator_callback) (iterator, NULL, fad->user_data);
 
-			dbus_error_init (&dbus_error);
-			dbus_set_error_from_message (&dbus_error, reply);
-			dbus_set_g_error (&error, &dbus_error);
-
-			(* fad->iterator_callback) (NULL, error, fad->user_data);
-
-			dbus_error_free (&dbus_error);
-		} else {
-			/* Call iterator callback */
-			fad->iterator_returned = TRUE;
-
-			(* fad->iterator_callback) (iterator, NULL, fad->user_data);
-		}
+		tracker_result_iterator_free (iterator);
 	} else {
 		if (error->code != G_IO_ERROR_CANCELLED) {
-			g_clear_error (&error);
-			g_set_error (&error,
-			             TRACKER_CLIENT_ERROR,
-			             TRACKER_CLIENT_ERROR_BROKEN_PIPE,
-			             "Couldn't get results from server");
+			GError *iterator_error;
 
-			(* fad->iterator_callback) (NULL, error, fad->user_data);
+			iterator_error = g_error_new (TRACKER_CLIENT_ERROR,
+			                              TRACKER_CLIENT_ERROR_BROKEN_PIPE,
+			                              "Couldn't get results from server");
+
+			(* fad->iterator_callback) (NULL, iterator_error, fad->user_data);
+
+			/* iterator_error was passed to the callback and should be
+			 * disposed there */
 		}
 
+		/* Always free input GError. We want to behave exactly as if this
+		 * callback were one used in an async dbus-glib query.  */
 		g_error_free (error);
 	}
 
-	/* Clean up */
-	if (reply) {
-		dbus_message_unref (reply);
-	}
-
 	fast_async_data_free (fad);
 }
 
@@ -752,6 +710,8 @@ callback_iterator_compat (GPtrArray *results,
 		iterator->current_row = -1;
 
 		(* data->callback) (iterator, error, data->user_data);
+
+		tracker_result_iterator_free (iterator);
 	}
 
 	g_slice_free (FastQueryAsyncCompatData, data);
@@ -853,6 +813,8 @@ is_service_available (void)
  * Escapes a string so it can be passed as a SPARQL parameter in
  * any query/update.
  *
+ * Deprecated: 0.10: For tracker_sparql_escape_string() in libtracker-sparql
+ *
  * Returns: the newly allocated escaped string which must be freed
  * using g_free().
  *
@@ -1087,6 +1049,7 @@ sparql_update_fast_callback (DBusPendingCall *call,
 
 		fast_async_data_free (fad);
 
+		dbus_pending_call_unref (call);
 		return;
 	}
 
@@ -1128,6 +1091,8 @@ sparql_update_fast_callback (DBusPendingCall *call,
 	dbus_message_unref (reply);
 
 	fast_async_data_free (fad);
+
+	dbus_pending_call_unref (call);
 }
 
 static DBusPendingCall *
@@ -1303,6 +1268,8 @@ sparql_update_fast_async (TrackerClient  *client,
  *
  * The result is escaped using g_uri_escape_string().
  *
+ * Deprecated: 0.10: For tracker_sparql_escape_uri_vprintf() in libtracker-sparql
+ *
  * Returns: a newly-allocated string holding the result which should
  * be freed with g_free() when finished with.
  *
@@ -1406,7 +1373,9 @@ cleanup:
  * @Varargs: the parameters to insert into the format string
  *
  * Calls tracker_uri_vprintf_escaped() with the @Varargs supplied.
-
+ *
+ * Deprecated: 0.10: For tracker_sparql_escape_uri_printf() in libtracker-sparql
+ *
  * Returns: a newly-allocated string holding the result which should
  * be freed with g_free() when finished with.
  *
@@ -1436,6 +1405,8 @@ tracker_uri_printf_escaped (const gchar *format, ...)
  * The @timeout is only used if it is > 0. If it is, then it is used
  * with dbus_g_proxy_set_default_timeout().
  *
+ * Deprecated: 0.10: For tracker_bus_connection_new() or tracker_direct_connection_new() in libtracker-sparql
+ *
  * Returns: the #TrackerClient which should be freed with
  * g_object_unref() when finished with.
  **/
@@ -1468,6 +1439,8 @@ tracker_client_new (TrackerClientFlags flags,
  * API call made using libtracker-client. For synchronous API calls,
  * see tracker_cancel_last_call() which is more useful.
  *
+ * Deprecated: 0.10: There is no replacement for this in libtracker-sparql
+ *
  * Returns: A @gboolean indicating if the call was cancelled or not.
  **/
 gboolean
@@ -1551,6 +1524,8 @@ tracker_cancel_call (TrackerClient *client,
  * last API call ID is always tracked so you don't have to provide it
  * with this API.
  *
+ * Deprecated: 0.10: There is no replacement for this in libtracker-sparql
+ *
  * Returns: A #gboolean indicating if the call was cancelled or not.
  **/
 gboolean
@@ -1586,6 +1561,8 @@ tracker_cancel_last_call (TrackerClient *client)
  *
  * This API call is completely synchronous so it may block.
  *
+ * Deprecated: 0.10: For tracker_sparql_stats() in libtracker-sparql
+ *
  * Returns: A #GPtrArray with the statistics which must be freed using
  * g_ptr_array_free().
  *
@@ -1611,6 +1588,18 @@ tracker_statistics_get (TrackerClient  *client,
 	return table;
 }
 
+/**
+ * tracker_resources_load:
+ * @client: a #TrackerClient.
+ * @uri: a string representing a Turtle file
+ * @error: a #GError.
+ *
+ * Imports SPARQL into the database from a file specified by @uri
+ *
+ * Deprecated: 0.10: For tracker_sparql_import() in libtracker-sparql
+ *
+ * Since: 0.8
+ **/
 void
 tracker_resources_load (TrackerClient  *client,
                         const gchar    *uri,
@@ -1678,6 +1667,8 @@ tracker_resources_load (TrackerClient  *client,
  *
  * This API call is completely synchronous so it may block.
  *
+ * Deprecated: 0.10: For tracker_sparql_query() in libtracker-sparql
+ *
  * Returns: A #GPtrArray with the query results which must be freed
  * using g_ptr_array_free().
  *
@@ -1766,33 +1757,30 @@ tracker_resources_sparql_query (TrackerClient  *client,
  * </programlisting>
  * </example>
  *
+ * Deprecated: 0.10: For tracker_sparql_query() in libtracker-sparql
+ *
  * Returns: A #TrackerResultIterator pointing before the first result row. This
  * iterator must be disposed when done using tracker_result_iterator_free().
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 TrackerResultIterator *
 tracker_resources_sparql_query_iterate (TrackerClient  *client,
                                         const gchar    *query,
                                         GError        **error)
 {
-	g_return_val_if_fail (TRACKER_IS_CLIENT (client), NULL);
-	g_return_val_if_fail (query, NULL);
-
 #ifdef HAVE_DBUS_FD_PASSING
 	TrackerClientPrivate *private;
 	TrackerResultIterator *iterator;
 	DBusConnection *connection;
 	DBusMessage *message;
 	DBusMessageIter iter;
-	DBusMessage *reply = NULL;
-	DBusPendingCall *call;
 	int pipefd[2];
-	GInputStream *input_stream;
-	GInputStream *buffered_input_stream;
-	GOutputStream *iterator_output_stream;
 	GError *inner_error = NULL;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), NULL);
+	g_return_val_if_fail (query, NULL);
+
 	if (pipe (pipefd) < 0) {
 		g_set_error (error,
 		             TRACKER_CLIENT_ERROR,
@@ -1813,84 +1801,34 @@ tracker_resources_sparql_query_iterate (TrackerClient  *client,
 	dbus_message_iter_init_append (message, &iter);
 	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &query);
 	dbus_message_iter_append_basic (&iter, DBUS_TYPE_UNIX_FD, &pipefd[1]);
-
-	dbus_connection_send_with_reply (connection,
-	                                 message,
-	                                 &call,
-	                                 -1);
-	dbus_message_unref (message);
 	close (pipefd[1]);
 
-	if (!call) {
-		g_set_error (error,
-		             TRACKER_CLIENT_ERROR,
-		             TRACKER_CLIENT_ERROR_UNSUPPORTED,
-		             "FD passing unsupported or connection disconnected");
-		return NULL;
-	}
 
-	input_stream = g_unix_input_stream_new (pipefd[0], TRUE);
-	buffered_input_stream = g_buffered_input_stream_new_sized (input_stream,
-	                                                           TRACKER_STEROIDS_BUFFER_SIZE);
-	iterator_output_stream = g_memory_output_stream_new (NULL, 0, g_realloc, NULL);
-
-	/* Reset the iterator internal state */
 	iterator = g_slice_new0 (TrackerResultIterator);
-	iterator->buffer_size = g_output_stream_splice (iterator_output_stream,
-	                                                buffered_input_stream,
-	                                                G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
-	                                                G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-	                                                NULL,
-	                                                &inner_error);
-	iterator->buffer = g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (iterator_output_stream));
-	iterator->buffer_index = 0;
 
-	/* Clean up streams */
-	g_object_unref (buffered_input_stream);
-	g_object_unref (iterator_output_stream);
-	g_object_unref (input_stream);
-
-	if (G_LIKELY (!inner_error)) {
-		/* Wait for any current d-bus call to finish */
-		dbus_pending_call_block (call);
-
-		/* Check we didn't get an error */
-		reply = dbus_pending_call_steal_reply (call);
-
-		if (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR) {
-			DBusError dbus_error;
-
-			dbus_error_init (&dbus_error);
-			dbus_set_error_from_message (&dbus_error, reply);
-			dbus_set_g_error (error, &dbus_error);
-			dbus_error_free (&dbus_error);
-
-			tracker_result_iterator_free (iterator);
-			iterator = NULL;
-		}
-	} else {
-		g_set_error (error,
-		             TRACKER_CLIENT_ERROR,
-		             TRACKER_CLIENT_ERROR_BROKEN_PIPE,
-		             "Couldn't get results from server");
-		g_error_free (inner_error);
+	tracker_dbus_send_and_splice (connection,
+	                              message,
+	                              pipefd[0],
+	                              NULL,
+	                              (void **) &iterator->buffer,
+	                              &iterator->buffer_size,
+	                              &inner_error);
+	/* message is destroyed by tracker_dbus_send_and_splice */
 
+	if (G_UNLIKELY (inner_error)) {
+		g_propagate_error (error, inner_error);
 		tracker_result_iterator_free (iterator);
 		iterator = NULL;
 	}
 
-	/* Clean up */
-	if (reply) {
-		dbus_message_unref (reply);
-	}
-
-	dbus_pending_call_unref (call);
-
 	return iterator;
 #else  /* HAVE_DBUS_FD_PASSING */
 	TrackerResultIterator *iterator;
 	GError *inner_error = NULL;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), NULL);
+	g_return_val_if_fail (query, NULL);
+
 	iterator = g_slice_new0 (TrackerResultIterator);
 
 	iterator->results = tracker_resources_sparql_query (client, query, &inner_error);
@@ -1912,7 +1850,9 @@ tracker_resources_sparql_query_iterate (TrackerClient  *client,
  *
  * Frees a TrackerResultIterator and its associated resources
  *
- * Since: 0.9
+ * Deprecated: 0.10: For g_object_unref() in libtracker-sparql
+ *
+ * Since: 0.10
  **/
 void
 tracker_result_iterator_free (TrackerResultIterator *iterator)
@@ -1932,21 +1872,26 @@ tracker_result_iterator_free (TrackerResultIterator *iterator)
  * tracker_result_iterator_n_columns:
  * @iterator: A TrackerResultIterator
  *
+ * Deprecated: 0.10: For tracker_cursor_n_columns() in libtracker-sparql
+ *
  * Returns: the number of columns in the row pointed by @iterator
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 guint
 tracker_result_iterator_n_columns (TrackerResultIterator *iterator)
 {
+#ifdef HAVE_DBUS_FD_PASSING
+
 	g_return_val_if_fail (iterator != NULL, 0);
 
-#ifdef HAVE_DBUS_FD_PASSING
 	return iterator->n_columns;
 #else  /* HAVE_DBUS_FD_PASSING */
 	GStrv row;
 	guint i = 0;
 
+	g_return_val_if_fail (iterator != NULL, 0);
+
 	if (!iterator->results->len) {
 		return 0;
 	}
@@ -1966,9 +1911,11 @@ tracker_result_iterator_n_columns (TrackerResultIterator *iterator)
  *
  * Fetches the next row for the results.
  *
+ * Deprecated: 0.10: For tracker_cursor_next() in libtracker-sparql
+ *
  * Returns: %TRUE if a rows was fetched, otherwise %FALSE.
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 gboolean
 tracker_result_iterator_next (TrackerResultIterator *iterator)
@@ -2016,16 +1963,18 @@ tracker_result_iterator_next (TrackerResultIterator *iterator)
  * Returns: the value of the column as a string. The returned string belongs to
  * the iterator and should not be freed.
  *
- * Since: 0.9
+ * Deprecated: 0.10: For tracker_cursor_get_string() in libtracker-sparql
+ *
+ * Since: 0.10
  **/
 const gchar *
 tracker_result_iterator_value (TrackerResultIterator *iterator,
                                guint                  column)
 {
+#ifdef HAVE_DBUS_FD_PASSING
 	g_return_val_if_fail (iterator != NULL, NULL);
 	g_return_val_if_fail (column < tracker_result_iterator_n_columns (iterator), NULL);
 
-#ifdef HAVE_DBUS_FD_PASSING
 	if (column == 0) {
 		return iterator->data;
 	} else {
@@ -2034,6 +1983,9 @@ tracker_result_iterator_value (TrackerResultIterator *iterator,
 #else  /* HAVE_DBUS_FD_PASSING */
 	GStrv row;
 
+	g_return_val_if_fail (iterator != NULL, NULL);
+	g_return_val_if_fail (column < tracker_result_iterator_n_columns (iterator), NULL);
+
 	if (!iterator->results->len) {
 		return NULL;
 	}
@@ -2060,6 +2012,8 @@ tracker_result_iterator_value (TrackerResultIterator *iterator,
  *
  * This API call is completely synchronous so it may block.
  *
+ * Deprecated: 0.10: For tracker_sparql_update() in libtracker-sparql
+ *
  * Since: 0.8
  **/
 void
@@ -2067,12 +2021,12 @@ tracker_resources_sparql_update (TrackerClient  *client,
                                  const gchar    *query,
                                  GError        **error)
 {
-	g_return_if_fail (TRACKER_IS_CLIENT (client));
-	g_return_if_fail (query != NULL);
-
 #ifdef HAVE_DBUS_FD_PASSING
 	DBusMessage *reply;
 
+	g_return_if_fail (TRACKER_IS_CLIENT (client));
+	g_return_if_fail (query != NULL);
+
 	reply = sparql_update_fast (client, query, FAST_UPDATE, error);
 
 	if (!reply) {
@@ -2083,6 +2037,9 @@ tracker_resources_sparql_update (TrackerClient  *client,
 #else  /* HAVE_DBUS_FD_PASSING */
 	TrackerClientPrivate *private;
 
+	g_return_if_fail (TRACKER_IS_CLIENT (client));
+	g_return_if_fail (query != NULL);
+
 	private = TRACKER_CLIENT_GET_PRIVATE (client);
 
 	org_freedesktop_Tracker1_Resources_sparql_update (private->proxy_resources,
@@ -2091,19 +2048,29 @@ tracker_resources_sparql_update (TrackerClient  *client,
 #endif /* HAVE_DBUS_FD_PASSING */
 }
 
+/**
+ * tracker_resources_sparql_update_blank:
+ * @client: a #TrackerClient.
+ * @query: a string representing SPARQL.
+ * @error: return location for errors.
+ *
+ * Deprecated: 0.10: There is no replacement for this in libtracker-sparql
+ *
+ * Since: 0.8
+ **/
 GPtrArray *
 tracker_resources_sparql_update_blank (TrackerClient  *client,
                                        const gchar    *query,
                                        GError        **error)
 {
-	g_return_val_if_fail (TRACKER_IS_CLIENT (client), NULL);
-	g_return_val_if_fail (query != NULL, NULL);
-
 #ifdef HAVE_DBUS_FD_PASSING
 	DBusMessage *reply;
 	DBusMessageIter iter, subiter, subsubiter;
 	GPtrArray *result;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), NULL);
+	g_return_val_if_fail (query != NULL, NULL);
+
 	reply = sparql_update_fast (client, query, FAST_UPDATE_BLANK, error);
 
 	if (!reply) {
@@ -2145,6 +2112,9 @@ tracker_resources_sparql_update_blank (TrackerClient  *client,
 	TrackerClientPrivate *private;
 	GPtrArray *result;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), NULL);
+	g_return_val_if_fail (query != NULL, NULL);
+
 	private = TRACKER_CLIENT_GET_PRIVATE (client);
 
 	if (!org_freedesktop_Tracker1_Resources_sparql_update_blank (private->proxy_resources,
@@ -2169,6 +2139,8 @@ tracker_resources_sparql_update_blank (TrackerClient  *client,
  * tracker_resources_batch_commit_async(). This API call is synchronous so it may
  * block.
  *
+ * Deprecated: 0.10: For tracker_sparql_update() in libtracker-sparql
+ *
  * Since: 0.8
  **/
 void
@@ -2208,6 +2180,8 @@ tracker_resources_batch_sparql_update (TrackerClient  *client,
  * Commits a batch of already issued SPARQL updates. This API call is
  * synchronous so it may block.
  *
+ * Deprecated: 0.10: For tracker_sparql_update_commit() in libtracker-sparql
+ *
  * Since: 0.8
  **/
 void
@@ -2233,6 +2207,8 @@ tracker_resources_batch_commit (TrackerClient  *client,
  *
  * This behaves exactly as tracker_statistics_get() but asynchronously.
  *
+ * Deprecated: 0.10: For tracker_sparql_stats_async() in libtracker-sparql
+ *
  * Returns: A #guint representing the operation ID. See
  * tracker_cancel_call(). In the event of failure, 0 is returned.
  *
@@ -2307,6 +2283,8 @@ tracker_resources_load_async (TrackerClient    *client,
  * Does an asynchronous SPARQL query. See tracker_resources_sparql_query()
  * to see how an SPARLQL query should be like.
  *
+ * Deprecated: 0.10: For tracker_sparql_query_async() in libtracker-sparql
+ *
  * Returns: A #guint representing the operation ID. See
  * tracker_cancel_call(). In the event of failure, 0 is returned.
  *
@@ -2343,6 +2321,20 @@ tracker_resources_sparql_query_async (TrackerClient         *client,
 	return cb->id;
 }
 
+/**
+ * tracker_resources_sparql_query_iterate_async
+ * @client: a #TrackerClient.
+ * @query: a string representing SPARQL.
+ * @callback: function to be called when the blank update has been performed.
+ * @user_data: user data to pass to @callback.
+ *
+ * Deprecated: 0.10: For tracker_sparql_query_async() in libtracker-sparql
+ *
+ * Returns: A #guint representing the operation ID. See
+ * tracker_cancel_call(). In the event of failure, 0 is returned.
+ *
+ * Since: 0.10
+ **/
 guint
 tracker_resources_sparql_query_iterate_async (TrackerClient         *client,
                                               const gchar           *query,
@@ -2354,11 +2346,7 @@ tracker_resources_sparql_query_iterate_async (TrackerClient         *client,
 	DBusConnection *connection;
 	DBusMessage *message;
 	DBusMessageIter iter;
-	DBusPendingCall *call;
 	int pipefd[2];
-	GInputStream *input_stream;
-	GInputStream *buffered_input_stream;
-	GOutputStream *iterator_output_stream;
 	GCancellable *cancellable;
 	FastAsyncData *fad;
 
@@ -2397,43 +2385,22 @@ tracker_resources_sparql_query_iterate_async (TrackerClient         *client,
 	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &query);
 	dbus_message_iter_append_basic (&iter, DBUS_TYPE_UNIX_FD, &pipefd[1]);
 
-	dbus_connection_send_with_reply (connection,
-	                                 message,
-	                                 &call,
-	                                 -1);
-	dbus_message_unref (message);
-
-	/* FIXME: Why do we close this? */
 	close (pipefd[1]);
 
-	if (!call) {
-		g_critical ("FD passing unsupported or connection disconnected");
-		return 0;
-	}
-
-	input_stream = g_unix_input_stream_new (pipefd[0], TRUE);
-	buffered_input_stream = g_buffered_input_stream_new_sized (input_stream,
-	                                                           TRACKER_STEROIDS_BUFFER_SIZE);
-	iterator_output_stream = g_memory_output_stream_new (NULL, 0, g_realloc, NULL);
 	cancellable = g_cancellable_new ();
 
 	fad = fast_async_data_new (client,
 	                           FAST_QUERY,
-	                           buffered_input_stream,
-	                           iterator_output_stream,
 	                           cancellable,
-	                           call,
 	                           user_data);
 	fad->iterator_callback = callback;
 
-	g_output_stream_splice_async (iterator_output_stream,
-	                              buffered_input_stream,
-	                              G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
-	                              G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-	                              0,
-	                              cancellable,
-	                              callback_iterator,
-	                              fad);
+	tracker_dbus_send_and_splice_async (connection,
+	                                    message,
+	                                    pipefd[0],
+	                                    cancellable,
+	                                    callback_iterator,
+	                                    fad);
 
 	return fad->request_id;
 #else  /* HAVE_DBUS_FD_PASSING */
@@ -2450,26 +2417,37 @@ tracker_resources_sparql_query_iterate_async (TrackerClient         *client,
 #endif /* HAVE_DBUS_FD_PASSING */
 }
 
+/**
+ * tracker_resources_sparql_update_async
+ * @client: a #TrackerClient.
+ * @query: a string representing SPARQL.
+ * @callback: function to be called when the blank update has been performed.
+ * @user_data: user data to pass to @callback.
+ *
+ * Deprecated: 0.10: For tracker_sparql_update_async() in libtracker-sparql
+ *
+ * Returns: A #guint representing the operation ID. See
+ * tracker_cancel_call(). In the event of failure, 0 is returned.
+ *
+ * Since: 0.8
+ **/
 guint
 tracker_resources_sparql_update_async (TrackerClient    *client,
                                        const gchar      *query,
                                        TrackerReplyVoid  callback,
                                        gpointer          user_data)
 {
-	g_return_val_if_fail (TRACKER_IS_CLIENT (client), 0);
-	g_return_val_if_fail (query != NULL, 0);
-	g_return_val_if_fail (callback != NULL, 0);
-
 #ifdef HAVE_DBUS_FD_PASSING
 	FastAsyncData *fad;
 	GError *error = NULL;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), 0);
+	g_return_val_if_fail (query != NULL, 0);
+	g_return_val_if_fail (callback != NULL, 0);
+
 	fad = fast_async_data_new (client,
 	                           FAST_UPDATE,
 	                           NULL,
-	                           NULL,
-	                           NULL,
-	                           NULL,
 	                           user_data);
 	fad->void_callback = callback;
 
@@ -2490,6 +2468,10 @@ tracker_resources_sparql_update_async (TrackerClient    *client,
 	CallbackVoid *cb;
 	DBusGProxyCall *call;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), 0);
+	g_return_val_if_fail (query != NULL, 0);
+	g_return_val_if_fail (callback != NULL, 0);
+
 	private = TRACKER_CLIENT_GET_PRIVATE (client);
 
 	cb = g_slice_new0 (CallbackVoid);
@@ -2508,26 +2490,37 @@ tracker_resources_sparql_update_async (TrackerClient    *client,
 #endif /* HAVE_DBUS_FD_PASSING */
 }
 
+/**
+ * tracker_resources_sparql_update_blank_async
+ * @client: a #TrackerClient.
+ * @query: a string representing SPARQL.
+ * @callback: function to be called when the blank update has been performed.
+ * @user_data: user data to pass to @callback.
+ *
+ * Deprecated: 0.10: There is no replacement for this in libtracker-sparql
+ *
+ * Returns: A #guint representing the operation ID. See
+ * tracker_cancel_call(). In the event of failure, 0 is returned.
+ *
+ * Since: 0.8
+ **/
 guint
 tracker_resources_sparql_update_blank_async (TrackerClient         *client,
                                              const gchar           *query,
                                              TrackerReplyGPtrArray  callback,
                                              gpointer               user_data)
 {
-	g_return_val_if_fail (TRACKER_IS_CLIENT (client), 0);
-	g_return_val_if_fail (query != NULL, 0);
-	g_return_val_if_fail (callback != NULL, 0);
-
 #ifdef HAVE_DBUS_FD_PASSING
 	FastAsyncData *fad;
 	GError *error = NULL;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), 0);
+	g_return_val_if_fail (query != NULL, 0);
+	g_return_val_if_fail (callback != NULL, 0);
+
 	fad = fast_async_data_new (client,
 	                           FAST_UPDATE_BLANK,
 	                           NULL,
-	                           NULL,
-	                           NULL,
-	                           NULL,
 	                           user_data);
 	fad->gptrarray_callback = callback;
 
@@ -2548,6 +2541,10 @@ tracker_resources_sparql_update_blank_async (TrackerClient         *client,
 	CallbackGPtrArray *cb;
 	DBusGProxyCall *call;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), 0);
+	g_return_val_if_fail (query != NULL, 0);
+	g_return_val_if_fail (callback != NULL, 0);
+
 	private = TRACKER_CLIENT_GET_PRIVATE (client);
 
 	cb = g_slice_new0 (CallbackGPtrArray);
@@ -2575,6 +2572,8 @@ tracker_resources_sparql_update_blank_async (TrackerClient         *client,
  *
  * Updates the database using SPARQL. see tracker_resources_batch_sparql_update().
  *
+ * Deprecated: 0.10: For tracker_sparql_update_async() in libtracker-sparql
+ *
  * Returns: A #guint representing the operation ID. See
  * tracker_cancel_call(). In the event of failure, 0 is returned.
  *
@@ -2586,20 +2585,17 @@ tracker_resources_batch_sparql_update_async (TrackerClient    *client,
                                              TrackerReplyVoid  callback,
                                              gpointer          user_data)
 {
-	g_return_val_if_fail (TRACKER_IS_CLIENT (client), 0);
-	g_return_val_if_fail (query != NULL, 0);
-	g_return_val_if_fail (callback != NULL, 0);
-
 #ifdef HAVE_DBUS_FD_PASSING
 	FastAsyncData *fad;
 	GError *error = NULL;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), 0);
+	g_return_val_if_fail (query != NULL, 0);
+	g_return_val_if_fail (callback != NULL, 0);
+
 	fad = fast_async_data_new (client,
 	                           FAST_UPDATE_BATCH,
 	                           NULL,
-	                           NULL,
-	                           NULL,
-	                           NULL,
 	                           user_data);
 	fad->void_callback = callback;
 
@@ -2620,6 +2616,10 @@ tracker_resources_batch_sparql_update_async (TrackerClient    *client,
 	CallbackVoid *cb;
 	DBusGProxyCall *call;
 
+	g_return_val_if_fail (TRACKER_IS_CLIENT (client), 0);
+	g_return_val_if_fail (query != NULL, 0);
+	g_return_val_if_fail (callback != NULL, 0);
+
 	private = TRACKER_CLIENT_GET_PRIVATE (client);
 
 	cb = g_slice_new0 (CallbackVoid);
@@ -2646,6 +2646,8 @@ tracker_resources_batch_sparql_update_async (TrackerClient    *client,
  *
  * Commits a batch of already issued SPARQL updates.
  *
+ * Deprecated: 0.10: For tracker_sparql_update_commit_async() in libtracker-sparql
+ *
  * Returns: A #guint representing the operation ID. See
  * tracker_cancel_call(). In the event of failure, 0 is returned.
  *
@@ -2694,6 +2696,8 @@ tracker_resources_batch_commit_async (TrackerClient    *client,
  * possible the changes in the RDF store should be reflected in the metadata of
  * the original file.
  *
+ * Deprecated: 0.10: There is no replacement for this in libtracker-sparql
+ *
  * Returns: a handle that can be used to disconnect the signal later using
  *          tracker_resources_writeback_disconnect. The handle will always be
  *          greater than 0 on success.
@@ -2736,6 +2740,8 @@ tracker_resources_writeback_connect (TrackerClient            *client,
  * @client: a #TrackerClient
  * @handle: a handle identifying a callback
  *
+ * Deprecated: 0.10: There is no replacement for this in libtracker-sparql
+ *
  * Removes the callback identified by @handle from the writeback callbacks.
  **/
 void
@@ -2806,8 +2812,6 @@ sparql_append_string_literal (GString     *sparql,
 	g_string_append_c (sparql, '"');
 }
 
-#ifndef TRACKER_DISABLE_DEPRECATED
-
 /**
  * tracker_connect:
  * @enable_warnings: a #gboolean to determine if warnings are issued in
