@@ -42,18 +42,17 @@
 #include <libtracker-common/tracker-log.h>
 #include <libtracker-common/tracker-ontologies.h>
 
-#include <libtracker-db/tracker-db-manager.h>
-#include <libtracker-db/tracker-db-dbus.h>
-
 #include <libtracker-data/tracker-data-manager.h>
 #include <libtracker-data/tracker-data-backup.h>
 #include <libtracker-data/tracker-data-query.h>
+#include <libtracker-data/tracker-db-config.h>
+#include <libtracker-data/tracker-db-dbus.h>
+#include <libtracker-data/tracker-db-manager.h>
 
 #include "tracker-dbus.h"
 #include "tracker-config.h"
 #include "tracker-events.h"
 #include "tracker-writeback.h"
-#include "tracker-push.h"
 #include "tracker-backup.h"
 #include "tracker-store.h"
 #include "tracker-statistics.h"
@@ -334,6 +333,11 @@ main (gint argc, gchar *argv[])
 	TrackerStatus *notifier;
 	gpointer busy_user_data;
 	TrackerBusyCallback busy_callback;
+	gint chunk_size_mb;
+	gsize chunk_size;
+	const gchar *rotate_to;
+	TrackerDBConfig *db_config;
+	gboolean do_rotating;
 
 	g_type_init ();
 
@@ -399,6 +403,7 @@ main (gint argc, gchar *argv[])
 
 	/* Initialize major subsystems */
 	config = tracker_config_new ();
+	db_config = tracker_db_config_new ();
 
 	g_signal_connect (config, "notify::verbosity",
 	                  G_CALLBACK (config_verbosity_changed_cb),
@@ -415,6 +420,7 @@ main (gint argc, gchar *argv[])
 	initialize_directories ();
 
 	if (!tracker_dbus_init ()) {
+		g_object_unref (db_config);
 		return EXIT_FAILURE;
 	}
 
@@ -438,6 +444,36 @@ main (gint argc, gchar *argv[])
 	busy_callback = tracker_status_get_callback (notifier,
 	                                            &busy_user_data);
 
+	tracker_store_init ();
+
+
+	/* Make Tracker available for introspection */
+	if (!tracker_dbus_register_objects ()) {
+		return EXIT_FAILURE;
+	}
+
+	if (!tracker_dbus_register_names ()) {
+		return EXIT_FAILURE;
+	}
+
+	chunk_size_mb = tracker_db_config_get_journal_chunk_size (db_config);
+	chunk_size = (gsize) ((gsize) chunk_size_mb * (gsize) 1024 * (gsize) 1024);
+	rotate_to = tracker_db_config_get_journal_rotate_destination (db_config);
+
+	if (rotate_to[0] == '\0')
+		rotate_to = NULL;
+
+	do_rotating = (chunk_size_mb != -1);
+
+	if (!GLIB_CHECK_VERSION (2, 24, 2)) {
+		if (do_rotating) {
+			g_warning ("Your GLib isn't recent enough for journal rotating to be enabled");
+			do_rotating = FALSE;
+		}
+	}
+
+	tracker_db_journal_set_rotating (do_rotating, chunk_size, rotate_to);
+
 	if (!tracker_data_manager_init (flags,
 	                                NULL,
 	                                &is_first_time_index,
@@ -446,27 +482,24 @@ main (gint argc, gchar *argv[])
 	                                busy_user_data,
 	                                "Journal replaying")) {
 
+		g_object_unref (db_config);
 		g_object_unref (notifier);
 		return EXIT_FAILURE;
 	}
 
+	g_object_unref (db_config);
 	g_object_unref (notifier);
 
-	tracker_store_init ();
-
 	if (private->shutdown) {
 		goto shutdown;
 	}
 
-	/* Make Tracker available for introspection */
-	if (!tracker_dbus_register_objects ()) {
-		return EXIT_FAILURE;
-	}
+	tracker_dbus_register_class_signal_objects ();
 
 	tracker_events_init (get_notifiable_classes);
 	tracker_writeback_init (get_writeback_predicates);
 
-	tracker_push_init ();
+	tracker_store_set_active (TRUE);
 
 	g_message ("Waiting for D-Bus requests...");
 
@@ -494,7 +527,6 @@ main (gint argc, gchar *argv[])
 	shutdown_directories ();
 
 	/* Shutdown major subsystems */
-	tracker_push_shutdown ();
 	tracker_writeback_shutdown ();
 	tracker_events_shutdown ();
 
@@ -505,6 +537,9 @@ main (gint argc, gchar *argv[])
 	g_signal_handlers_disconnect_by_func (config, config_verbosity_changed_cb, NULL);
 	g_object_unref (config);
 
+	/* This will free rotate_to up in the journal code */
+	tracker_db_journal_set_rotating ((chunk_size_mb != -1), chunk_size, NULL);
+
 	g_print ("\nOK\n\n");
 
 	return EXIT_SUCCESS;
