@@ -29,11 +29,10 @@
 #include <libtracker-common/tracker-type-utils.h>
 #include <libtracker-common/tracker-ontologies.h>
 
-#include <libtracker-db/tracker-db-dbus.h>
-
 #include <libtracker-data/tracker-data-manager.h>
 #include <libtracker-data/tracker-data-query.h>
 #include <libtracker-data/tracker-data-update.h>
+#include <libtracker-data/tracker-db-dbus.h>
 
 #include "tracker-dbus.h"
 #include "tracker-marshal.h"
@@ -57,7 +56,7 @@
  * bytes plus DBusMessage's overhead. If that makes this number less
  * arbitrary for you, then fine.
  *
- * I really hope that the libdbus people get to their senses and 
+ * I really hope that the libdbus people get to their senses and
  * either stop doing their exit() nonsense in a library, and instead
  * return a clean DBusError or something, or create crystal clear
  * clarity about the maximum size of a message. And make it both so
@@ -184,6 +183,7 @@ tracker_resources_load (TrackerResources         *object,
 	TrackerDBusMethodInfo   *info;
 	guint                    request_id;
 	GFile  *file;
+	gchar *sender;
 
 	request_id = tracker_dbus_get_next_request_id ();
 
@@ -202,9 +202,12 @@ tracker_resources_load (TrackerResources         *object,
 	info->request_id = request_id;
 	info->context = context;
 
-	tracker_store_queue_turtle_import (file, turtle_import_callback,
-	                                   info, destroy_method_info);
+	sender = dbus_g_method_get_sender (context);
 
+	tracker_store_queue_turtle_import (file, turtle_import_callback,
+	                                   sender, info, destroy_method_info);
+
+	g_free (sender);
 	g_object_unref (file);
 }
 
@@ -227,19 +230,19 @@ query_callback (gpointer inthread_data, GError *error, gpointer user_data)
 		                             &error,
 		                             NULL);
 		dbus_g_method_return_error (info->context, error);
-	} else {
+	} else if (ptr) {
 		tracker_dbus_request_success (info->request_id,
 		                              info->context);
 
 		dbus_g_method_send_reply (info->context, ptr->reply);
-	}
+	} /* else, !ptr && !error... shouldn't happen */
 
 	if (ptr)
 		g_slice_free (InThreadPtr, ptr);
 }
 
 static gpointer
-query_inthread (TrackerDBCursor *cursor, GError *error, gpointer user_data)
+query_inthread (TrackerDBCursor *cursor, GCancellable *cancellable, GError *error, gpointer user_data)
 {
 	InThreadPtr *ptr = g_slice_new0 (InThreadPtr);
 	TrackerDBusMethodInfo *info = user_data;
@@ -268,7 +271,7 @@ query_inthread (TrackerDBCursor *cursor, GError *error, gpointer user_data)
 
 	cont = TRUE;
 
-	while (tracker_db_cursor_iter_next (cursor, &loop_error) && cont) {
+	while (tracker_db_cursor_iter_next (cursor, cancellable, &loop_error) && cont) {
 		DBusMessageIter cols_iter;
 		guint i;
 
@@ -397,7 +400,7 @@ tracker_resources_sparql_update (TrackerResources        *self,
 
 	sender = dbus_g_method_get_sender (context);
 
-	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_HIGH, FALSE,
+	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_HIGH,
 	                             update_callback, sender,
 	                             info, destroy_method_info);
 
@@ -504,53 +507,19 @@ tracker_resources_batch_sparql_update (TrackerResources          *self,
 
 	sender = dbus_g_method_get_sender (context);
 
-	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_LOW, TRUE,
+	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_LOW,
 	                             update_callback, sender,
 	                             info, destroy_method_info);
 
 	g_free (sender);
 }
 
-static void
-batch_commit_callback (gpointer user_data)
-{
-	TrackerDBusMethodInfo *info = user_data;
-
-	tracker_data_sync ();
-
-	tracker_dbus_request_success (info->request_id,
-	                              info->context);
-
-	dbus_g_method_return (info->context);
-}
-
 void
 tracker_resources_batch_commit (TrackerResources         *self,
                                 DBusGMethodInvocation    *context,
                                 GError                  **error)
 {
-	TrackerDBusMethodInfo *info;
-	guint                 request_id;
-	gchar                 *sender;
-
-	request_id = tracker_dbus_get_next_request_id ();
-
-	tracker_dbus_request_new (request_id,
-	                          context,
-	                          "%s()",
-	                          __FUNCTION__);
-
-	info = g_slice_new (TrackerDBusMethodInfo);
-
-	info->request_id = request_id;
-	info->context = context;
-
-	sender = dbus_g_method_get_sender (context);
-
-	tracker_store_queue_commit (batch_commit_callback, sender, info,
-	                            destroy_method_info);
-
-	g_free (sender);
+	/* no longer needed */
 }
 
 
